/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
/** 
	 * Screenshot.java
	 * Autor: Alberto Poblacin Navarro
	 * Aadido al proyecto en la version v0.1.5
	 * Aadido a:
	 		* Proyecto original: http://code.google.com/p/android-vnc-viewer
	 		* Release: 39
	 */
package android.androidVNC;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Calendar;
import java.util.GregorianCalendar;

import android.app.AlertDialog;
import android.graphics.Bitmap;
import android.os.Environment;
import android.widget.Toast;

public class Screenshot {
	/**
	 * Realiza un screenshot de lo que vemos actualmente en pantalla
	 * @param vncCanvasActivity instancia de esta clase utilizada
	 * @param vncCanvas instancia de esta clase utilizada
	 * @return true si se realiza el screenshot correctamente, false en caso contrario
	 * @author Alberto Poblacin Navarro
	 */
	static public boolean nuevo(VncCanvasActivity vncCanvasActivity, VncCanvas vncCanvas){
		
		int num_imagen=0;
		boolean retorno = true;
		
		String path;
		String sdcard_status = Environment.getExternalStorageState();
		File carpeta;
		//Comprobamos si hay un medio extraible donde se pueda guardar la imagen
        if(sdcard_status.equals(Environment.MEDIA_REMOVED)){
        	//Si no hay tarjeta de memoria creamos la carpeta en la memoria interna del movil
        	//Se crea en la carpeta del propio programa
        	path = "/data/data/android.androidVNC/screenshot/";
        }else path = "/sdcard/screenshot/"; //Si hay tarjeta se guarda en la tarjeta
         carpeta=new File(path);
		if(!carpeta.exists())
		  carpeta.mkdir(); //Si la carpeta no existe la creamos
			  
		  
		 int dia=0;
		 int mes=0;
		 int year=0;
		 try{
			  dia = new GregorianCalendar().get(Calendar.DAY_OF_MONTH);
			  mes = (new GregorianCalendar().get(Calendar.MONTH))+1; //No obtiene bien el mes
			  year = new GregorianCalendar().get(Calendar.YEAR);
		 }catch (Exception e) {
			if (e instanceof IllegalArgumentException){
				Toast.makeText(vncCanvasActivity, vncCanvasActivity.getString(R.string.no_date), Toast.LENGTH_SHORT).show();
				dia=mes=year=0;

			}
		 }
		 String nombre = new String("screenshot"+dia+mes+year+".jpeg");
		  
		  //Archivo que usara el buffer de salida para guardar la imagen
		  File imagen = new File(path+nombre); 
		  while(imagen.exists())
		  {
			  nombre= new String("screenshot"+dia+mes+year+"("+num_imagen+")"+".jpeg");
			  imagen= new File(path+nombre);
			  num_imagen++;
		  }
		  num_imagen=0;
		  Bitmap nueva=null;
		  //Coje solo lo que vemos en la pantalla del movil aplicando el mismo zoom que se est aplicando
		  try{
			  nueva = Bitmap.createBitmap(vncCanvas.bitmapData.mbitmap,
				 		(int)(vncCanvasActivity.absoluteXPosition/Zoom.getZoom()),
				 		(int)(vncCanvasActivity.absoluteYPosition/Zoom.getZoom()),
						(int)(vncCanvas.getWidth()/Zoom.getZoom()), 
						(int)(vncCanvas.getHeight()/Zoom.getZoom()), 
						Zoom.getMatrix(), true);
		  }catch (Exception e) {
			  new AlertDialog.Builder(vncCanvasActivity)
	          .setTitle(vncCanvasActivity.getString(R.string.error))
	          .setMessage(vncCanvasActivity.getString(R.string.cause)+": "+e.toString())
	          .setIcon(android.R.drawable.ic_dialog_alert)
	          .setPositiveButton(android.R.string.ok, null)
	          .show();
			retorno=false;
		  }
		  //Guarda la imagen en formato JPEG y muestra un mensaje confirmandolo
		  try{
			  FileOutputStream fos= new FileOutputStream(imagen);
			  nueva.compress(Bitmap.CompressFormat.JPEG,100,fos);
			  fos.flush();
			  fos.close();
			  Toast.makeText(vncCanvasActivity, vncCanvasActivity.getString(R.string.screenshot)+": "+ nombre, Toast.LENGTH_SHORT).show();
			   
			  /** Error de SDK, no deja guardarlo, devuelve null siempre
			  ContentResolver cr = vncCanvasActivity.getContentResolver();
			  MediaStore.Images.Media.insertImage(cr,path+nombre, nombre,"descricion"))
				*/			  
		  }catch (Exception e) {
			  //En caso de error mostramos cuadro de dialogo con el error
			  	  new AlertDialog.Builder(vncCanvasActivity)
		          .setTitle(vncCanvasActivity.getString(R.string.error))
		          .setMessage(vncCanvasActivity.getString(R.string.cause)+": "+e.toString())
		          .setIcon(android.R.drawable.ic_dialog_alert)
		          .setPositiveButton(android.R.string.ok, null)
		          .show();
				retorno=false;
			}finally{
				nueva.recycle(); //Una vez guardada la imagen la liberamos
			}
		  
		  if(retorno)
			  Notificacion.Instancia(vncCanvasActivity).notificar(vncCanvasActivity.getString(R.string.screenshot),vncCanvasActivity.getString(R.string.image_saved)+": "+path+nombre);
		  
		  
		  return retorno;
	}
}
